@@ -2,12 +2,12 @@
 # this repository contains the full copyright notices and license terms.
 from trytond import backend
 from trytond.i18n import gettext
-from trytond.model import ModelView, ModelSQL, ModelSingleton, fields
-from trytond.model import MultiValueMixin, ValueMixin
+from trytond.model import (
+    ModelSingleton, ModelSQL, ModelView, MultiValueMixin, ValueMixin, fields)
 from trytond.model.exceptions import AccessError
 from trytond.pool import Pool
-from trytond.tools.multivalue import migrate_property
 from trytond.pyson import Id
+from trytond.tools.multivalue import migrate_property
 
 from .party import IDENTIFIER_TYPES
 
@@ -65,16 +65,19 @@ class Configuration(ModelSingleton, ModelSQL, ModelView, MultiValueMixin):
         ModelView._fields_view_get_cache.clear()
 
     @classmethod
-    def validate(cls, records):
-        super().validate(records)
-        cls(1).check_identifier_types()
+    def validate_fields(cls, records, field_names):
+        super().validate_fields(records, field_names)
+        cls(1).check_identifier_types(field_names)
 
-    def check_identifier_types(self):
+    def check_identifier_types(self, field_names=None):
         pool = Pool()
         Identifier = pool.get('party.identifier')
+        if field_names and 'identifier_types' not in field_names:
+            return
         if self.identifier_types:
+            identifier_types = [None, ''] + list(self.identifier_types)
             identifiers = Identifier.search([
-                    ('type', 'not in', [None, ''] + self.identifier_types),
+                    ('type', 'not in', identifier_types),
                     ], limit=1, order=[])
             if identifiers:
                 identifier, = identifiers
