@@ -1,15 +1,17 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 from itertools import chain
+
 try:
     import phonenumbers
-    from phonenumbers import PhoneNumberFormat, NumberParseException
+    from phonenumbers import NumberParseException, PhoneNumberFormat
 except ImportError:
     phonenumbers = None
 
 from trytond.i18n import gettext
 from trytond.model import (
-    ModelView, ModelSQL, DeactivableMixin, fields, sequence_ordered)
+    DeactivableMixin, ModelSQL, ModelView, MultiValueMixin, ValueMixin, fields,
+    sequence_ordered)
 from trytond.model.exceptions import AccessError
 from trytond.pyson import Eval
 from trytond.transaction import Transaction
@@ -37,7 +39,8 @@ _PHONE_TYPES = {
 
 
 class ContactMechanism(
-        DeactivableMixin, sequence_ordered(), ModelSQL, ModelView):
+        DeactivableMixin, sequence_ordered(), ModelSQL, ModelView,
+        MultiValueMixin):
     "Contact Mechanism"
     __name__ = 'party.contact_mechanism'
     _rec_name = 'value'
@@ -45,38 +48,45 @@ class ContactMechanism(
     type = fields.Selection(_TYPES, "Type", required=True, sort=False)
     value = fields.Char("Value", select=True,
         # Add all function fields to ensure to always fill them via on_change
-        depends=[
+        depends={
             'email', 'website', 'skype', 'sip', 'other_value',
-            'value_compact'])
+            'value_compact'})
     value_compact = fields.Char('Value Compact', readonly=True)
     name = fields.Char("Name")
     comment = fields.Text("Comment")
     party = fields.Many2One(
         'party.party', "Party", required=True, ondelete='CASCADE', select=True)
+    language = fields.MultiValue(
+        fields.Many2One('ir.lang', "Language",
+            help="Used to translate communication made "
+            "using the contact mechanism.\n"
+            "Leave empty for the party language."))
+    languages = fields.One2Many(
+        'party.contact_mechanism.language', 'contact_mechanism', "Languages")
     email = fields.Function(fields.Char('E-Mail', states={
         'invisible': Eval('type') != 'email',
         'required': Eval('type') == 'email',
-        }, depends=['value', 'type']),
+        }, depends={'value'}),
         'get_value', setter='set_value')
     website = fields.Function(fields.Char('Website', states={
         'invisible': Eval('type') != 'website',
         'required': Eval('type') == 'website',
-        }, depends=['value', 'type']),
+        }, depends={'value'}),
         'get_value', setter='set_value')
     skype = fields.Function(fields.Char('Skype', states={
         'invisible': Eval('type') != 'skype',
         'required': Eval('type') == 'skype',
-        }, depends=['value', 'type']),
+        }, depends={'value'}),
         'get_value', setter='set_value')
     sip = fields.Function(fields.Char('SIP', states={
         'invisible': Eval('type') != 'sip',
         'required': Eval('type') == 'sip',
-        }, depends=['value', 'type']),
+        }, depends={'value'}),
         'get_value', setter='set_value')
     other_value = fields.Function(fields.Char('Value', states={
         'invisible': Eval('type').in_(['email', 'website', 'skype', 'sip']),
         'required': ~Eval('type').in_(['email', 'website']),
-        }, depends=['value', 'type']),
+        }, depends={'value'}),
         'get_value', setter='set_value')
     url = fields.Function(fields.Char('URL', states={
                 'invisible': ~Eval('url'),
@@ -86,7 +96,8 @@ class ContactMechanism(
     @classmethod
     def __setup__(cls):
         super(ContactMechanism, cls).__setup__()
-        cls._order.insert(0, ('party', 'ASC'))
+        cls._order.insert(0, ('party.distance', 'ASC NULLS LAST'))
+        cls._order.insert(1, ('party', 'ASC'))
 
     @staticmethod
     def default_type():
@@ -131,9 +142,12 @@ class ContactMechanism(
         for country_code in chain(self._phone_country_codes(), [None]):
             try:
                 # Country code is ignored if value has an international prefix
-                return phonenumbers.parse(value, country_code)
+                phonenumber = phonenumbers.parse(value, country_code)
             except NumberParseException:
                 pass
+            else:
+                if phonenumbers.is_valid_number(phonenumber):
+                    return phonenumber
         return None
 
     @fields.depends(methods=['_parse_phonenumber'])
@@ -248,19 +262,23 @@ class ContactMechanism(
         cls._format_values(all_mechanisms)
 
     @classmethod
-    def validate(cls, mechanisms):
-        super(ContactMechanism, cls).validate(mechanisms)
-        for mechanism in mechanisms:
-            mechanism.check_valid_phonenumber()
+    def validate_fields(cls, mechanisms, field_names):
+        super().validate_fields(mechanisms, field_names)
+        cls.check_valid_phonenumber(mechanisms, field_names)
 
-    def check_valid_phonenumber(self):
-        if not phonenumbers or self.type not in _PHONE_TYPES:
+    @classmethod
+    def check_valid_phonenumber(cls, mechanisms, field_names=None):
+        if field_names and not (field_names & {'type', 'value'}):
             return
-        phonenumber = self._parse_phonenumber(self.value)
-        if not (phonenumber and phonenumbers.is_valid_number(phonenumber)):
-            raise InvalidPhoneNumber(
-                gettext('party.msg_invalid_phone_number',
-                    phone=self.value, party=self.party.rec_name))
+        if not phonenumbers:
+            return
+        for mechanism in mechanisms:
+            if mechanism.type not in _PHONE_TYPES:
+                continue
+            if not mechanism._parse_phonenumber(mechanism.value):
+                raise InvalidPhoneNumber(
+                    gettext('party.msg_invalid_phone_number',
+                        phone=mechanism.value, party=mechanism.party.rec_name))
 
     @classmethod
     def usages(cls, _fields=None):
@@ -270,3 +288,12 @@ class ContactMechanism(
             for name, desc in cls.fields_get(_fields).items():
                 usages.append((name, desc['string']))
         return usages
+
+
+class ContactMechanismLanguage(ModelSQL, ValueMixin):
+    "Contact Mechanism Language"
+    __name__ = 'party.contact_mechanism.language'
+    contact_mechanism = fields.Many2One(
+        'party.contact_mechanism', "Contact Mechanism",
+        ondelete='CASCADE', select=True)
+    language = fields.Many2One('ir.lang', "Language")
